require_dependency "skirental_frontend_api/application_controller"

module SkirentalFrontendApi
    class BookingsController < ApplicationController

        def context
            puts "SkirentalFrontendApi::BookingsController.context"
            booking = Booking.where(shop_id: booking_params[:shop_id], session_id: booking_params[:session_id]).first
            if booking.present?
                if booking.context.present?
                    booking.context[:changeableUntil] = booking.changeable_until
                end
                if booking.modified_at.present?
                    cleared_cart = false
                    if booking.payments.blank? && !booking.fetch_is_checkin()
                        expiration_time = booking.modified_at + booking.ghost_time * 60
                        if Time.now > expiration_time
                            puts "GHOST_BOOKING_EXPIRED! LOOK WHETHER WE NEED TO CLEAR THE CART"
                            cleared_cart = booking.clear_cart()
                            if cleared_cart
                                puts "…YES, WE HAD TO CLEAR THE CART"
                                booking.modified_at = Time.now
                                booking.cleared_cart = true
                                booking.no_sync = true
                                booking.save
                            end
                        end
                    end
                    unless cleared_cart
                        reset_cleared_cart(booking)
                    end
                end
                render json: booking.serializable_hash(include: :payments)
            else
                render json: booking
            end
        end

        def create
            puts "SkirentalFrontendApi::BookingsController.create"
            # check whether booking was already created. This should not happen, but..
            new_booking = Booking.where(shop_id: booking_params[:shop_id], session_id: booking_params[:session_id]).first
            if new_booking.nil?
                new_booking = Booking.new(shop_id: booking_params[:shop_id], session_id: booking_params[:session_id], context: booking_params[:booking].to_h)
                if booking_params[:creation_time].present?
                    new_booking.creation_time = booking_params[:creation_time]
                end
            else
                render json: { error: "sessionId already exists" }, :status => :unprocessable_entity
                return
            end
            if new_booking.save
                ghost_valid_time = new_booking.modified_at + new_booking.ghost_time * 60
                render json: { error: "", booking_id: new_booking.booking_id, voucher: new_booking.voucher, ghost_valid_time: ghost_valid_time, booking: new_booking.serializable_hash(include: :payments) }, :status => :created
            else
                render json: { error: new_booking.errors.full_messages }, :status => :unprocessable_entity
            end
        end

        def updatability
            puts "SkirentalFrontendApi::BookingsController.updatability"
            booking = Booking.where(shop_id: booking_params[:shop_id], session_id: booking_params[:session_id]).first
            api_booking = SkirentalApi::Client.get_booking(booking.booking_id)
            if (api_booking[:editUntil] != booking.changeable_until)
                booking.changeable_until = api_booking[:editUntil]
                booking.no_sync = true
                booking.save
            end
            puts "***\n***\n***\nBOOKING UPDATABILITY for id '#{booking.booking_id}': #{booking.changeable_until}\n***\n***\n***"
            render json: { changeable_until: api_booking[:editUntil] }
        end

        def signature_state
            puts "SkirentalFrontendApi::BookingsController.signature_state"
            booking = Booking.where(shop_id: booking_params[:shop_id], session_id: booking_params[:session_id]).first
            if booking.nil?
                puts "booking is nil :("
                render json: { error: "not found" }, :status => :not_found
            else
                signature_state = SkirentalApi::Client.get_booking_signature(booking.context[:shopLic], booking.booking_id, booking.context[:locale])
                render json: signature_state
            end
        end

        def signature_invalidation
            puts "SkirentalFrontendApi::BookingsController.signature_invalidation"
            booking = Booking.where(shop_id: booking_params[:shop_id], session_id: booking_params[:session_id]).first
            if booking.nil?
                puts "booking is nil :("
                render json: { error: "not found" }, :status => :not_found
            else
                reponse = SkirentalApi::Client.invalidate_signature(booking.context[:shopLic], booking.booking_id)
                render json: reponse, :status => :ok
            end
        end

        def signing_url
            puts "SkirentalFrontendApi::BookingsController.signing_url"
            booking = Booking.where(shop_id: booking_params[:shop_id], session_id: booking_params[:session_id]).first
            if booking.nil?
                puts "booking is nil"
                render json: { error: "not found" }, :status => :not_found
            else
                api_booking = SkirentalApi::Client.get_booking(booking.booking_id)
                render json: { signingUrl: api_booking[:signingUrl] }
            end
        end

        def update
            puts "SkirentalFrontendApi::BookingsController.update"
            booking = Booking.where(shop_id: booking_params[:shop_id], session_id: booking_params[:session_id]).first

            if booking.nil?
                render json: { error: "not found" }, :status => :not_found
            else
                unless booking.already_finished
                    puts "BOOKING FOUND, TRYING TO SAVE UPDATED CONTEXT"
                    booking.context = booking_params[:booking].to_h
                    puts "#{booking.context}"
                    if booking.save
                        ghost_valid_time = booking.modified_at + booking.ghost_time * 60
                        render json: {
                            error: booking.errors.present? ? booking.errors.full_messages : "",
                            booking_id: booking.booking_id,
                            voucher: booking.voucher,
                            ghost_valid_time: ghost_valid_time,
                            booking: booking.serializable_hash(include: :payments)
                        }, :status => :ok
                    else
                        render json: { error: booking.errors.full_messages }, :status => :unprocessable_entity
                    end
                else
                    # don't allow everything -> only customer.personalData and product.selectedVariant can be updated from now on…
                    puts "BOOKING ALREADY PAYED/RESERVED!!"
                    begin
                        # TODO: Update for new structure
                        updated_context = booking_params[:booking].to_h

                        # update booking.code once if not set yet (directly after payment)
                        if booking.context["code"].blank? and updated_context["code"].present?
                            booking.context["code"] = updated_context["code"]
                        end

                        # update customer notice and arrivalDate
                        booking.context["customerNotice"] = updated_context["customerNotice"]
                        booking.context["arrivalDate"] = updated_context["arrivalDate"]

                        # update followedEmailLink variable
                        booking.context["followedEmailLink"] = updated_context["followedEmailLink"]
        
                        # Iterate over updated customers
                        updated_context["customers"].each_with_index do |customer, customer_index|
                            # Overwrite personalData in persisted context
                            booking.context["customers"][customer_index]["personalData"] = customer["personalData"]

                            # Overwrite selectedVariants in persisted context
                            customer["packages"].each_with_index do |package, package_index|
                                booking.context["customers"][customer_index]["packages"][package_index]["mainProduct"]["selectedVariant"] = package["mainProduct"]["selectedVariant"]
                                package["accessories"].each_with_index do |accessorie, accessorie_index|
                                    booking.context["customers"][customer_index]["packages"][package_index]["accessories"][accessorie_index]["selectedVariant"] = accessorie["selectedVariant"]
                                end
                            end
                            customer["products"].each_with_index do |product, product_index|
                                booking.context["customers"][customer_index]["products"][product_index]["selectedVariant"] = product["selectedVariant"]
                            end
                        end

                        if booking_params[:no_sync].to_s.present?
                            if booking_params[:no_sync].to_s == 'true'
                                booking.no_sync = true
                            end
                        end

                        if booking.save
                            render json: { error: "", booking_id: booking.booking_id, voucher: booking.voucher, booking: booking.serializable_hash(include: :payments) }, :status => :ok
                        else
                            render json: { error: booking.errors.full_messages }, :status => :unprocessable_entity
                        end
                    rescue Exception => e
                        # most probably the lengths of customers and/or products arrays didn't match…
                        render json: { error: "already paid or reserved: #{e}" }, :status => :unprocessable_entity
                    end
                end
            end
        end

        def accept
            puts "SkirentalFrontendApi::BookingsController.accept"
            booking = Booking.where(shop_id: booking_params[:shop_id], session_id: booking_params[:session_id]).first

            if booking.context["code"].present?
                response = SkirentalApi::Client.accept_booking(booking.context["code"])
                render json: { error: nil, response: response }, :status => :ok
            else
                render json: { error: "no booking code available" }, :status => :unprocessable_entity
            end
        end

        private

        def booking_params
            params.permit(:shop_id, :session_id, :no_sync, :creation_time, booking: {})
        end

        def reset_cleared_cart(booking)
            if booking.cleared_cart == true
                booking.cleared_cart = false
                booking.no_sync = true
                booking.save
            end
        end
    end
end
